@@ -1,5 +1,6 @@
 /*
     Copyright (c) 2019 David Edmundson <davidedmundson@kde.org>
+    Copyright (c) 2020 David Redondo <kde@david-redondo.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -19,6 +20,8 @@
 
 #include "ksysguarddaemon.h"
 
+#include <chrono>
+
 #include <QDBusArgument>
 #include <QDBusConnection>
 #include <QDBusMessage>
@@ -32,6 +35,7 @@
 #include "SensorContainer.h"
 #include "SensorProperty.h"
 
+#include <KDBusService>
 #include <KPluginLoader>
 #include <KPluginMetaData>
 #include <KPluginFactory>
@@ -39,6 +43,8 @@
 #include "ksysguard_ifaceadaptor.h"
 #include "client.h"
 
+constexpr auto UpdateRate = std::chrono::milliseconds{500};
+
 KSysGuardDaemon::KSysGuardDaemon()
     : m_serviceWatcher(new QDBusServiceWatcher(this))
 {
@@ -55,7 +61,7 @@ KSysGuardDaemon::KSysGuardDaemon()
     connect(m_serviceWatcher, &QDBusServiceWatcher::serviceUnregistered, this, &KSysGuardDaemon::onServiceDisconnected);
 
     auto timer = new QTimer(this);
-    timer->setInterval(2000);
+    timer->setInterval(UpdateRate);
     connect(timer, &QTimer::timeout, this, &KSysGuardDaemon::sendFrame);
     timer->start();
 }
@@ -67,14 +73,16 @@ KSysGuardDaemon::~KSysGuardDaemon()
     }
 }
 
-void KSysGuardDaemon::init()
+void KSysGuardDaemon::init(ReplaceIfRunning replaceIfRunning)
 {
     loadProviders();
-    QDBusConnection::sessionBus().registerObject("/", this, QDBusConnection::ExportAdaptors);
-    if (!QDBusConnection::sessionBus().registerService("org.kde.ksystemstats")) {
-        qCritical() << "Unable to register DBus service org.kde.ksystemstats. Maybe it is already running?";
-        exit(1);
+    KDBusService::StartupOptions options = KDBusService::Unique;
+    if (replaceIfRunning == ReplaceIfRunning::Replace) {
+        options |= KDBusService::Replace;
     }
+    QDBusConnection::sessionBus().registerObject("/", this, QDBusConnection::ExportAdaptors);
+    auto service = new KDBusService(options , this);
+    service->setExitValue(1);
 }
 
 void KSysGuardDaemon::loadProviders()
